@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2021 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -20,16 +20,18 @@
  *
  */
 
-#include <core/LocalFileMng.h>
+#include <core/Basics/Drumkit.h>
 #include <core/config.h>
+#include <core/EventQueue.h>
 #include <core/Helpers/Filesystem.h>
 #include <core/Hydrogen.h>
+#include <core/SoundLibrary/SoundLibraryDatabase.h>
 
 #include <QtCore/QDir>
 #include <QtCore/QFile>
 #include <QtCore/QFileInfo>
 #include <QtCore/QCoreApplication>
-#include <QDomDocument>
+#include <QDateTime>
 
 #ifdef H2CORE_HAVE_OSC
 #include <core/NsmClient.h>
@@ -49,6 +51,7 @@
 #define REPOSITORIES    "repositories/"
 #define SCRIPTS         "scripts/"
 #define SONGS           "songs/"
+#define THEMES          "themes/"
 #define TMP             "hydrogen/"
 #define XSD             "xsd/"
 
@@ -57,38 +60,42 @@
 /** Sound of metronome beat */
 #define CLICK_SAMPLE    "click.wav"
 #define EMPTY_SAMPLE    "emptySample.wav"
-#define EMPTY_SONG      "DefaultSong.h2song"
+#define DEFAULT_SONG    "DefaultSong"
+#define EMPTY_SONG_BASE "emptySong"
 #define USR_CONFIG		"hydrogen.conf"
 #define SYS_CONFIG		"hydrogen.default.conf"
 #define LOG_FILE		"hydrogen.log"
 #define DRUMKIT_XML     "drumkit.xml"
 #define DRUMKIT_XSD     "drumkit.xsd"
 #define DRUMPAT_XSD     "drumkit_pattern.xsd"
+#define DRUMKIT_DEFAULT_KIT "GMRockKit"
 #define PLAYLIST_XSD     "playlist.xsd"
 
 #define AUTOSAVE        "autosave"
 
-#define UNTITLED_SONG		"untitled.h2song"
+#define UNTITLED_SONG		"Untitled Song"
 #define UNTITLED_PLAYLIST	"untitled.h2playlist"
 
 // filters
 #define PATTERN_FILTER  "*.h2pattern"
 #define PLAYLIST_FILTER "*.h2playlist"
 #define SONG_FILTER     "*.h2song"
+#define THEME_FILTER     "*.h2theme"
 
 namespace H2Core
 {
 
 Logger* Filesystem::__logger = nullptr;
-const char* Filesystem::__class_name = "Filesystem";
 
 const QString Filesystem::scripts_ext = ".sh";
 const QString Filesystem::songs_ext = ".h2song";
+const QString Filesystem::themes_ext = ".h2theme";
 const QString Filesystem::patterns_ext = ".h2pattern";
 const QString Filesystem::playlist_ext = ".h2playlist";
 const QString Filesystem::drumkit_ext = ".h2drumkit";
 const QString Filesystem::scripts_filter_name = "Hydrogen Scripts (*.sh)";
 const QString Filesystem::songs_filter_name = "Hydrogen Songs (*.h2song)";
+const QString Filesystem::themes_filter_name = "Hydrogen Theme (*.h2theme)";
 const QString Filesystem::patterns_filter_name = "Hydrogen Patterns (*.h2pattern)";
 const QString Filesystem::playlists_filter_name = "Hydrogen Playlists (*.h2playlist)";
 
@@ -187,33 +194,47 @@ bool Filesystem::check_permissions( const QString& path, const int perms, bool s
 	if( ( perms & is_file ) && ( perms & is_writable ) && !fi.exists() ) {
 		QFileInfo folder( path.left( path.lastIndexOf( "/" ) ) );
 		if( !folder.isDir() ) {
-			if( !silent ) ERRORLOG( QString( "%1 is not a directory" ).arg( folder.fileName() ) );
+			if( !silent ) {
+				ERRORLOG( QString( "%1 is not a directory" ).arg( folder.fileName() ) );
+			}
 			return false;
 		}
 		if( !folder.isWritable() ) {
-			if( !silent ) ERRORLOG( QString( "%1 is not writable" ).arg( folder.fileName() ) );
+			if( !silent ) {
+				ERRORLOG( QString( "%1 is not writable" ).arg( folder.fileName() ) );
+			}
 			return false;
 		}
 		return true;
 	}
 	if( ( perms & is_dir ) && !fi.isDir() ) {
-		if( !silent ) ERRORLOG( QString( "%1 is not a directory" ).arg( path ) );
+		if( !silent ) {
+			ERRORLOG( QString( "%1 is not a directory" ).arg( path ) );
+		}
 		return false;
 	}
 	if( ( perms & is_file ) && !fi.isFile() ) {
-		if( !silent ) ERRORLOG( QString( "%1 is not a file" ).arg( path ) );
+		if( !silent ) {
+			ERRORLOG( QString( "%1 is not a file" ).arg( path ) );
+		}
 		return false;
 	}
 	if( ( perms & is_readable ) && !fi.isReadable() ) {
-		if( !silent ) ERRORLOG( QString( "%1 is not readable" ).arg( path ) );
+		if( !silent ) {
+			ERRORLOG( QString( "%1 is not readable" ).arg( path ) );
+		}
 		return false;
 	}
 	if( ( perms & is_writable ) && !fi.isWritable() ) {
-		if( !silent ) ERRORLOG( QString( "%1 is not writable" ).arg( path ) );
+		if( !silent ) {
+			ERRORLOG( QString( "%1 is not writable" ).arg( path ) );
+		}
 		return false;
 	}
 	if( ( perms & is_executable ) && !fi.isExecutable() ) {
-		if( !silent ) ERRORLOG( QString( "%1 is not executable" ).arg( path ) );
+		if( !silent ) {
+			ERRORLOG( QString( "%1 is not executable" ).arg( path ) );
+		}
 		return false;
 	}
 	return true;
@@ -235,6 +256,10 @@ bool Filesystem::file_executable( const QString& path, bool silent )
 {
 	return check_permissions( path, is_file|is_executable, silent );
 }
+bool Filesystem::dir_exists(  const QString& path, bool silent )
+{
+	return check_permissions( path, is_dir, silent );
+}
 bool Filesystem::dir_readable(  const QString& path, bool silent )
 {
 	return check_permissions( path, is_dir|is_readable|is_executable, silent );
@@ -255,10 +280,14 @@ bool Filesystem::mkdir( const QString& path )
 
 bool Filesystem::path_usable( const QString& path, bool create, bool silent )
 {
-	if( !QDir( path ).exists() ) {
-		if( !silent ) INFOLOG( QString( "create user directory : %1" ).arg( path ) );
-		if( create && !QDir( "/" ).mkpath( path ) ) {
-			if( !silent ) ERRORLOG( QString( "unable to create user directory : %1" ).arg( path ) );
+	if ( !QDir( path ).exists() ) {
+		if ( !silent ) {
+			INFOLOG( QString( "create user directory : %1" ).arg( path ) );
+		}
+		if ( create && !QDir( "/" ).mkpath( path ) ) {
+			if( !silent ) {
+				ERRORLOG( QString( "unable to create user directory : %1" ).arg( path ) );
+			}
 			return false;
 		}
 	}
@@ -282,7 +311,7 @@ bool Filesystem::write_to_file( const QString& dst, const QString& content )
 	return true;
 }
 
-bool Filesystem::file_copy( const QString& src, const QString& dst, bool overwrite )
+bool Filesystem::file_copy( const QString& src, const QString& dst, bool overwrite, bool bSilent )
 {
 	if( !overwrite && file_exists( dst, true ) ) {
 		WARNINGLOG( QString( "do not overwrite %1 with %2 as it already exists" ).arg( dst ).arg( src ) );
@@ -296,19 +325,20 @@ bool Filesystem::file_copy( const QString& src, const QString& dst, bool overwri
 		ERRORLOG( QString( "unable to copy %1 to %2, %2 is not writable" ).arg( src ).arg( dst ) );
 		return false;
 	}
-	INFOLOG( QString( "copy %1 to %2" ).arg( src ).arg( dst ) );
-
+	if ( ! bSilent ) {
+		INFOLOG( QString( "copy %1 to %2" ).arg( src ).arg( dst ) );
+	}
 	
 	// Since QFile::copy does not overwrite, we have to make sure the
 	// destination does not exist.
 	if ( overwrite && file_exists( dst, true ) ) {
-		rm( dst, true );
+		rm( dst, true, bSilent );
 	}
 	
 	return QFile::copy( src, dst );
 }
 
-bool Filesystem::rm( const QString& path, bool recursive )
+bool Filesystem::rm( const QString& path, bool recursive, bool bSilent )
 {
 	if ( check_permissions( path, is_file, true ) ) {
 		QFile file( path );
@@ -330,18 +360,22 @@ bool Filesystem::rm( const QString& path, bool recursive )
 		}
 		return ret;
 	}
-	return rm_fr( path );
+	return rm_fr( path, bSilent );
 }
 
-bool Filesystem::rm_fr( const QString& path )
+bool Filesystem::rm_fr( const QString& path, bool bSilent )
 {
+	if ( ! bSilent ) {
+		INFOLOG( QString( "Removing [%1] recursively" ).arg( path ) );
+	}
+	
 	bool ret = true;
 	QDir dir( path );
 	QFileInfoList entries = dir.entryInfoList( QDir::NoDotAndDotDot | QDir::AllEntries );
 	for ( int idx = 0; ( ( idx < entries.size() ) && ret ); idx++ ) {
 		QFileInfo entryInfo = entries[idx];
 		if ( entryInfo.isDir() && !entryInfo.isSymLink() ) {
-			ret = rm_fr( entryInfo.absoluteFilePath() );
+			ret = rm_fr( entryInfo.absoluteFilePath(), bSilent );
 		} else {
 			QFile file( entryInfo.absoluteFilePath() );
 			if ( !file.remove() ) {
@@ -362,7 +396,6 @@ bool Filesystem::check_sys_paths()
 	bool ret = true;
 	if(  !dir_readable( __sys_data_path ) ) ret = false;
 	if( !file_readable( click_file_path() ) ) ret = false;
-	if( !file_readable( empty_song_path() ) ) ret = false;
 	if(  !dir_readable( demos_dir() ) ) ret = false;
 	/* if(  !dir_readable( doc_dir() ) ) ret = false; */		// FIXME
 	if(  !dir_readable( sys_drumkits_dir() ) ) ret = false;
@@ -370,12 +403,16 @@ bool Filesystem::check_sys_paths()
 	if( !file_readable( sys_config_path() ) ) ret = false;
 	if(  !dir_readable( i18n_dir() ) ) ret = false;
 	if(  !dir_readable( img_dir() ) ) ret = false;
+	if(  !dir_readable( sys_theme_dir() ) ) ret = false;
 	if(  !dir_readable( xsd_dir() ) ) ret = false;
 	if( !file_readable( pattern_xsd_path() ) ) ret = false;
 	if( !file_readable( drumkit_xsd_path() ) ) ret = false;
 	if( !file_readable( playlist_xsd_path() ) ) ret = false;
 
-	if ( ret ) INFOLOG( QString( "system wide data path %1 is usable." ).arg( __sys_data_path ) );
+	if ( ret ) {
+		INFOLOG( QString( "system wide data path %1 is usable." ).arg( __sys_data_path ) );
+	}
+	
 	return ret;
 }
 
@@ -393,9 +430,14 @@ bool Filesystem::check_usr_paths()
 	if( !path_usable( plugins_dir() ) ) ret = false;
 	if( !path_usable( scripts_dir() ) ) ret = false;
 	if( !path_usable( songs_dir() ) ) ret = false;
+	if( file_exists( empty_song_path(), true ) ) ret = false;
+	if( !path_usable( usr_theme_dir() ) ) ret = false;
 	if( !file_writable( usr_config_path() ) ) ret = false;
 
-	if ( ret ) INFOLOG( QString( "user path %1 is usable." ).arg( __usr_data_path ) );
+	if ( ret ) {
+		INFOLOG( QString( "user path %1 is usable." ).arg( __usr_data_path ) );
+	}
+	
 	return ret;
 }
 
@@ -426,11 +468,31 @@ QString Filesystem::empty_sample_path()
 {
 	return __sys_data_path + EMPTY_SAMPLE;
 }
-QString Filesystem::empty_song_path()
-{
-	return __sys_data_path + EMPTY_SONG;
+
+QString Filesystem::default_song_name() {
+	return DEFAULT_SONG;
 }
-QString Filesystem::untitled_song_file_name()
+
+QString Filesystem::empty_song_path() {
+	QString sPathBase( __usr_data_path + EMPTY_SONG_BASE );
+	QString sPath( sPathBase + Filesystem::songs_ext );
+
+	int nIterations = 0;
+	while ( file_exists( sPath, true ) ) {
+		sPath = sPathBase + QString::number( nIterations ) + Filesystem::songs_ext;
+		++nIterations;
+
+		if ( nIterations > 1000 ) {
+			ERRORLOG( "That's a bit much. Something is wrong in here." );
+			return __usr_data_path + SONGS + default_song_name() +
+				Filesystem::songs_ext;
+		}
+	}
+
+	return sPath;
+}
+
+QString Filesystem::untitled_song_name()
 {
 	return UNTITLED_SONG;
 }
@@ -447,6 +509,10 @@ QString Filesystem::usr_click_file_path()
 	if( file_readable( __usr_data_path + CLICK_SAMPLE, true ) ) return __usr_data_path + CLICK_SAMPLE;
 	return click_file_path();
 }
+QString Filesystem::drumkit_xsd( )
+{
+	return DRUMKIT_XSD;
+}
 QString Filesystem::drumkit_xsd_path( )
 {
 	return xsd_dir() + DRUMKIT_XSD;
@@ -485,6 +551,14 @@ QString Filesystem::songs_dir()
 {
 	return __usr_data_path + SONGS;
 }
+QString Filesystem::usr_theme_dir()
+{
+	return __usr_data_path + THEMES;
+}
+QString Filesystem::sys_theme_dir()
+{
+	return __sys_data_path + THEMES;
+}
 QString Filesystem::song_path( const QString& sg_name )
 {
 	return QString( songs_dir() + sg_name + songs_ext );
@@ -541,18 +615,26 @@ QString Filesystem::xsd_dir()
 {
 	return __sys_data_path + XSD;
 }
+QString Filesystem::xsd_legacy_dir()
+{
+	return xsd_dir() + "legacy";
+}
 QString Filesystem::tmp_dir()
 {
 	return QDir::tempPath() + "/" + TMP;
 }
-QString Filesystem::tmp_file_path( const QString& base )
+QString Filesystem::tmp_file_path( const QString &base )
 {
-	QFileInfo f( base );
-	QString templateName(tmp_dir() + "/");
+	// Ensure template base will produce a valid filename
+	QString validBase = base;
+	validBase.remove( QRegExp( "[^a-zA-Z0-9._]" ) );
+
+	QFileInfo f( validBase );
+	QString templateName( tmp_dir() + "/" );
 	if ( f.suffix().isEmpty() ) {
-		templateName += base;
+		templateName += validBase.left( 20 );
 	} else {
-		templateName += f.completeBaseName() + "-XXXXXX." + f.suffix();
+		templateName += f.completeBaseName().left( 20 ) + "-XXXXXX." + f.suffix();
 	}
 	QTemporaryFile file( templateName);
 	file.setAutoRemove( false );
@@ -575,6 +657,33 @@ QStringList Filesystem::drumkit_list( const QString& path )
 	}
 	return ok;
 }
+QString Filesystem::drumkit_default_kit() {
+	QString sDefaultPath = sys_drumkits_dir() + DRUMKIT_DEFAULT_KIT;
+
+	// GMRockKit does not exist at system-level? Let's pick another
+	// one.
+	if ( ! drumkit_valid( sDefaultPath ) ) {
+		for ( const auto& sDrumkitName : Filesystem::sys_drumkit_list() ) {
+			if ( drumkit_valid( Filesystem::sys_drumkits_dir() + sDrumkitName ) ) {
+				sDefaultPath = Filesystem::sys_drumkits_dir() + sDrumkitName;
+				break;
+			}
+		}
+	}
+
+	// There is no drumkit at system-level? Let's pick one from user-space.
+	if ( ! drumkit_valid( sDefaultPath ) ) {
+		for ( const auto& sDrumkitName : Filesystem::usr_drumkit_list() ) {
+			if ( drumkit_valid( Filesystem::usr_drumkits_dir() + sDrumkitName ) ) {
+				sDefaultPath = Filesystem::usr_drumkits_dir() + sDrumkitName;
+				break;
+			}
+		}
+	}
+
+	return sDefaultPath;
+}
+
 QStringList Filesystem::sys_drumkit_list( )
 {
 	return drumkit_list( sys_drumkits_dir() ) ;
@@ -645,7 +754,7 @@ QString Filesystem::drumkit_path_search( const QString& dk_name, Lookup lookup,
 	if ( Hydrogen::get_instance()->isUnderSessionManagement() ) {
 		
 		QString sDrumkitPath = QString( "%1/%2" )
-			.arg( NsmClient::get_instance()->m_sSessionFolderPath )
+			.arg( NsmClient::get_instance()->getSessionFolderPath() )
 			.arg( "drumkit" );
 		
 		// If the path is symbolic link, dereference it.
@@ -658,38 +767,32 @@ QString Filesystem::drumkit_path_search( const QString& dk_name, Lookup lookup,
 		// drumkit (using its name).
 		QString sDrumkitXMLPath = QString( "%1/%2" )
 				.arg( sDrumkitPath ).arg( "drumkit.xml" );
-		QFileInfo drumkitXMLInfo( sDrumkitXMLPath );
-		if ( drumkitXMLInfo.exists() ) {
-	
-			QDomDocument drumkitXML = H2Core::LocalFileMng::openXmlDocument( sDrumkitXMLPath );
-			QDomNodeList nodeList = drumkitXML.elementsByTagName( "drumkit_info" );
-	
-			if( nodeList.isEmpty() && ! bSilent ) {
-				NsmClient::printError( "Local drumkit does not seem valid" );
-			} else {
-				QDomNode drumkitInfoNode = nodeList.at( 0 );
-				QString sDrumkitNameXML = H2Core::LocalFileMng::readXmlString( drumkitInfoNode, "name", "" );
-	
-				if ( sDrumkitNameXML == dk_name ) {
-					// Jackpot. The local drumkit seems legit.	
-					return sDrumkitPath;
-					
-				} else {
-					if ( ! bSilent ) {
-						NsmClient::printError( QString( "Local drumkit [%1] and the one referenced in the .h2song file [%2] do not match!" )
-											   .arg( sDrumkitNameXML )
-											   .arg( dk_name ) );
-					}
-				}
+
+		QString sSessionDrumkitName( "seemsLikeTheKitCouldNotBeRetrievedFromTheDatabase" );
+		auto pSoundLibraryDatabase = Hydrogen::get_instance()->getSoundLibraryDatabase();
+		if ( pSoundLibraryDatabase != nullptr ) {
+			auto pDrumkit = pSoundLibraryDatabase->getDrumkit( sDrumkitPath );
+			if ( pDrumkit != nullptr ) {
+				sSessionDrumkitName = pDrumkit->get_name();
 			}
 		}
 
+		if ( dk_name == sSessionDrumkitName ) {
+				// The local drumkit seems legit.	
+				return sDrumkitPath;
+		}
+		else if ( ! bSilent ) {
+			NsmClient::printError( QString( "Local drumkit [%1] name [%2] and the one stored in .h2song file [%3] do not match!" )
+								   .arg( sDrumkitXMLPath )
+								   .arg( sSessionDrumkitName )
+								   .arg( dk_name ) );
+		}
 	}
 			
 #endif
 	
 	if ( lookup == Lookup::stacked || lookup == Lookup::user ) {
-		if( usr_drumkit_list().contains( dk_name ) ){
+		if ( usr_drumkit_list().contains( dk_name ) ){
 			return usr_drumkits_dir() + dk_name;
 		}
 	}
@@ -701,13 +804,14 @@ QString Filesystem::drumkit_path_search( const QString& dk_name, Lookup lookup,
 	}
 
 	if ( ! bSilent ) {
-		ERRORLOG( QString( "drumkit %1 not found using lookup type [%2]" )
+		ERRORLOG( QString( "drumkit [%1] not found using lookup type [%2]" )
 				  .arg( dk_name )
 				  .arg( static_cast<int>(lookup)));
 	}
 	
 	return QString("");
 }
+	
 QString Filesystem::drumkit_dir_search( const QString& dk_name, Lookup lookup )
 {
 	if ( lookup == Lookup::user || lookup == Lookup::stacked ) {
@@ -726,6 +830,31 @@ QString Filesystem::drumkit_dir_search( const QString& dk_name, Lookup lookup )
 }
 bool Filesystem::drumkit_valid( const QString& dk_path )
 {
+#ifdef H2CORE_HAVE_OSC
+	auto pHydrogen = Hydrogen::get_instance();
+	if ( pHydrogen != nullptr &&
+		 pHydrogen->isUnderSessionManagement() ) {
+
+		// Explicit handling for relative drumkit paths supported in
+		// the session management.
+		QFileInfo info( dk_path );
+		if ( info.isRelative() ) {
+			QString sAbsoluteDrumkitPath = QString( "%1%2" )
+				.arg( NsmClient::get_instance()->getSessionFolderPath() )
+				// remove the leading dot indicating that the path is relative. 
+				.arg( dk_path.right( dk_path.size() - 1 ) );
+
+			QFileInfo infoAbs( sAbsoluteDrumkitPath );
+			if ( infoAbs.isSymLink() ) {
+				sAbsoluteDrumkitPath = infoAbs.symLinkTarget();
+			}
+
+			return file_readable( sAbsoluteDrumkitPath + "/" +
+								  DRUMKIT_XML, true );
+		}
+	}
+#endif
+		
 	return file_readable( dk_path + "/" + DRUMKIT_XML, true);
 }
 QString Filesystem::drumkit_file( const QString& dk_path )
@@ -733,6 +862,15 @@ QString Filesystem::drumkit_file( const QString& dk_path )
 	return dk_path + "/" + DRUMKIT_XML;
 }
 
+QString Filesystem::drumkit_xml() {
+	return DRUMKIT_XML;
+}
+
+QString Filesystem::drumkit_backup_path( const QString& dk_path ) {
+	return dk_path + "." +
+		QDateTime::currentDateTime().toString( "yyyy-MM-dd_hh-mm-ss" ) + ".bak";
+}
+
 // PATTERNS
 QStringList Filesystem::pattern_drumkits()
 {
@@ -771,6 +909,57 @@ bool Filesystem::song_exists( const QString& sg_name )
 	return QDir( songs_dir() ).exists( sg_name );
 }
 
+bool Filesystem::isSongPathValid( const QString& sSongPath, bool bCheckExistance ) {
+	
+	QFileInfo songFileInfo = QFileInfo( sSongPath );
+
+	if ( !songFileInfo.isAbsolute() ) {
+		ERRORLOG( QString( "Error: Unable to handle path [%1]. Please provide an absolute file path!" )
+						.arg( sSongPath.toLocal8Bit().data() ));
+		return false;
+	}
+	
+	if ( songFileInfo.exists() ) {
+		if ( !songFileInfo.isReadable() ) {
+			ERRORLOG( QString( "Unable to handle path [%1]. You must have permissions to read the file!" )
+						.arg( sSongPath.toLocal8Bit().data() ));
+			return false;
+		}
+		if ( !songFileInfo.isWritable() ) {
+			WARNINGLOG( QString( "You don't have permissions to write to the Song found in path [%1]. It will be opened as read-only (no autosave)." )
+						.arg( sSongPath.toLocal8Bit().data() ));
+			EventQueue::get_instance()->push_event( EVENT_UPDATE_SONG, 2 );
+		}
+	} else if ( bCheckExistance ) {
+		ERRORLOG( QString( "Provided song [%1] does not exist" ).arg( sSongPath ) );
+		return false;
+	}
+	
+	if ( songFileInfo.suffix() != "h2song" ) {
+		ERRORLOG( QString( "Unable to handle path [%1]. The provided file must have the suffix '.h2song'!" )
+					.arg( sSongPath.toLocal8Bit().data() ));
+		return false;
+	}
+	
+	return true;
+}
+
+QString Filesystem::validateFilePath( const QString& sPath ) {
+
+	// Ensure the name will be a valid filename
+	QString sValidName( sPath );
+	sValidName.replace( " ", "_" );
+	sValidName.remove( QRegExp( "[^a-zA-Z0-9_-]" ) );
+
+	return sValidName;
+}
+
+QStringList Filesystem::theme_list( )
+{
+	return QDir( sys_theme_dir() ).entryList( QStringList( THEME_FILTER ), QDir::Files | QDir::Readable | QDir::NoDotAndDotDot ) +
+		QDir( usr_theme_dir() ).entryList( QStringList( THEME_FILTER ), QDir::Files | QDir::Readable | QDir::NoDotAndDotDot );
+}
+
 // PLAYLISTS
 QStringList Filesystem::playlist_list( )
 {
@@ -808,6 +997,71 @@ void Filesystem::info()
 	INFOLOG( QString( "Songs dir                  : %1" ).arg( songs_dir() ) );
 }
 
+QString Filesystem::absolute_path( const QString& sFilename, bool bSilent ) {
+	if ( QFile( sFilename ).exists() ) {
+		return QFileInfo( sFilename ).absoluteFilePath();
+	}
+	else if ( ! bSilent ) {
+		___ERRORLOG( QString( "File [%1] not found" ).arg( sFilename ) );
+	}
+
+	return QString();
+}
+
+QString Filesystem::ensure_session_compatibility( const QString& sPath ) {
+#ifdef H2CORE_HAVE_OSC
+	auto pHydrogen = Hydrogen::get_instance();
+	if ( pHydrogen != nullptr &&
+		 pHydrogen->isUnderSessionManagement() ) {
+
+		QFileInfo info( sPath );
+		if ( info.isRelative() ) {
+			return QString( "%1%2" )
+				.arg( NsmClient::get_instance()->getSessionFolderPath() )
+				// remove the leading dot indicating that the path is relative. 
+				.arg( sPath.right( sPath.size() - 1 ) );
+		}
+	}
+#endif
+
+	return sPath;
+}
+
+Filesystem::DrumkitType Filesystem::determineDrumkitType( const QString& sPath ) {
+	if ( sPath.contains( Filesystem::sys_drumkits_dir() ) ) {
+		return DrumkitType::System;
+	}
+	else if ( sPath.contains( Filesystem::usr_drumkits_dir() ) ) {
+		return DrumkitType::User;
+	}
+	else {
+		if ( dir_writable( sPath, true ) ) {
+			return DrumkitType::SessionReadWrite;
+		} else {
+			return DrumkitType::SessionReadOnly;
+		}
+	}
+}
+
+QStringList Filesystem::drumkit_xsd_legacy_paths() {
+	const QDir legacyDir( xsd_legacy_dir() );
+
+	const QStringList legacyDirSubfolders =
+		legacyDir.entryList( QDir::Dirs | QDir::NoDotAndDotDot,
+							 QDir::Name | QDir::Reversed );
+
+	QStringList drumkitXSDs;
+	for ( const auto& ffolder : legacyDirSubfolders ) {
+		const QDir folder( legacyDir.filePath( ffolder ) );
+		
+		if ( folder.exists( drumkit_xsd() ) ) {
+			drumkitXSDs << folder.filePath( drumkit_xsd() );
+		}
+	}
+
+	return std::move( drumkitXSDs );
+}
+
 };
 
 /* vim: set softtabstop=4 noexpandtab: */
