@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2021 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -23,22 +23,24 @@
 #define EVENT_LISTENER
 
 #include <core/Globals.h>
+#include <core/AudioEngine/AudioEngine.h>
+/** \ingroup docGUI docEvent*/
 class EventListener
 {
 	public:
-		virtual void stateChangedEvent(int nState) { UNUSED( nState ); }
-		virtual void patternChangedEvent() {}
+	virtual void stateChangedEvent( H2Core::AudioEngine::State state) { UNUSED( state ); }
+	virtual void playingPatternsChangedEvent() {}
+	virtual void nextPatternsChangedEvent(){}
 		virtual void patternModifiedEvent() {}
 		virtual void songModifiedEvent() {}
 		virtual void selectedPatternChangedEvent() {}
 		virtual void selectedInstrumentChangedEvent() {}
-		virtual void parametersInstrumentChangedEvent() {}
+	virtual void instrumentParametersChangedEvent( int nInstrumentNumber ) { UNUSED( nInstrumentNumber ); }
 		virtual void midiActivityEvent() {}
 		virtual void noteOnEvent( int nInstrument ) { UNUSED( nInstrument ); }
 		virtual void XRunEvent() {}
 		virtual void errorEvent( int nErrorCode ) { UNUSED( nErrorCode ); }
 		virtual void metronomeEvent( int nValue ) { UNUSED( nValue ); }
-		virtual void rubberbandbpmchangeEvent() {}
 		virtual void progressEvent( int nValue ) { UNUSED( nValue ); }
 		virtual void jacksessionEvent( int nValue) { UNUSED( nValue ); }
 		virtual void playlistLoadSongEvent( int nIndex ){ UNUSED( nIndex ); }
@@ -46,14 +48,25 @@ class EventListener
 		virtual void tempoChangedEvent( int nValue ){ UNUSED( nValue ); }
 		virtual void updateSongEvent( int nValue ){ UNUSED( nValue ); }
 		virtual void quitEvent( int nValue ){ UNUSED( nValue ); }
-		virtual void timelineActivationEvent( int nValue ){ UNUSED( nValue ); }
+		virtual void timelineActivationEvent(){}
 		virtual void timelineUpdateEvent( int nValue ){ UNUSED( nValue ); }
-		virtual void jackTransportActivationEvent( int nValue ){ UNUSED( nValue ); }
-		virtual void jackTimebaseActivationEvent( int nValue ){ UNUSED( nValue ); }
-		virtual void songModeActivationEvent( int nValue ){ UNUSED( nValue ); }
-		virtual void loopModeActivationEvent( int nValue ){ UNUSED( nValue ); }
+		virtual void jackTransportActivationEvent(){}
+		virtual void jackTimebaseStateChangedEvent(){}
+		virtual void songModeActivationEvent(){}
+		virtual void stackedModeActivationEvent( int nValue ){ UNUSED( nValue ); }
+		virtual void loopModeActivationEvent(){}
 		virtual void updatePreferencesEvent( int nValue ){ UNUSED( nValue ); }
 		virtual void actionModeChangeEvent( int nValue ){ UNUSED( nValue ); }
+    	virtual void gridCellToggledEvent(){}
+	virtual void drumkitLoadedEvent(){}
+	virtual void patternEditorLockedEvent(){}
+	virtual void relocationEvent(){}
+	virtual void bbtChangedEvent(){}
+	virtual void songSizeChangedEvent(){}
+	virtual void driverChangedEvent(){}
+	virtual void playbackTrackChangedEvent(){}
+	virtual void soundLibraryChangedEvent(){}
+	virtual void nextShotEvent(){}
 
 		virtual ~EventListener() {}
 };
