@@ -1,6 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
+ * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -15,30 +16,25 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program. If not, see https://www.gnu.org/licenses
  *
  *
  */
 
 #include "VirtualPatternDialog.h"
 
-#include <hydrogen/hydrogen.h>
-#include <hydrogen/basics/song.h>
-#include <hydrogen/basics/pattern.h>
-#include <hydrogen/basics/pattern_list.h>
-
-#include "Skin.h"
-
-const char* VirtualPatternDialog::__class_name = "VirtualPatternDialog";
+#include <core/Hydrogen.h>
+#include <core/Basics/Song.h>
+#include <core/Basics/Pattern.h>
+#include <core/Basics/PatternList.h>
 
 VirtualPatternDialog::VirtualPatternDialog(QWidget* parent)
     : QDialog(parent)
-    , Object( __class_name )
 {
     setupUi( this );
-    setFixedSize( width(), height() );
     setWindowTitle( tr( "Select virtual pattern" ) );
+    adjustSize();
+    setMinimumSize( width(), height() );
 }
 
 VirtualPatternDialog::~VirtualPatternDialog()
