@@ -1,6 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
+ * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -15,8 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program. If not, see https://www.gnu.org/licenses
  *
  */
 
@@ -24,7 +24,8 @@
 #ifndef MIDILEARNABLE_H
 #define MIDILEARNABLE_H
 
-#include <hydrogen/midi_action.h>
+#include <memory>
+#include <core/MidiAction.h>
 
 
 
@@ -32,28 +33,25 @@
   Every widget which supports MidiLearn should derive from this Class.
 */
 
+/** \ingroup docGUI docWidgets docMIDI*/
 class MidiLearnable
 {
 public:
     MidiLearnable(){
-	m_action = nullptr;
+		m_pAction = nullptr;
+	}
+
+    void setAction( std::shared_ptr<Action> pAction ){
+		m_pAction = pAction;
     }
 
-    ~MidiLearnable(){
-	if( m_action != nullptr) delete m_action;
-    }
-
-    void setAction( Action *action ){
-	m_action = action;
-    }
-
-    Action* getAction(){
-	return m_action;
+    std::shared_ptr<Action> getAction() const {
+		return m_pAction;
     }
 
 
-private:
-    Action *m_action;
+protected:
+    std::shared_ptr<Action> m_pAction;
 };
 
 #endif // MIDILEARNABLE_H
