@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2021 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -26,6 +26,7 @@
 
 #include <core/config.h>
 #include <core/Version.h>
+#include <core/Preferences/Theme.h>
 #include <getopt.h>
 
 #include "ShotList.h"
@@ -34,24 +35,28 @@
 #include "MainForm.h"
 #include "PlaylistEditor/PlaylistDialog.h"
 #include "Skin.h"
+#include "Reporter.h"
 
 #ifdef H2CORE_HAVE_LASH
 #include <core/Lash/LashClient.h>
 #endif
-#ifdef H2CORE_HAVE_JACKSESSION
-#include <jack/session.h>
+
+#ifdef WIN32
+#include <windows.h>
+#include <stdio.h>
 #endif
 
 #include <core/MidiMap.h>
-#include <core/AudioEngine.h>
+#include <core/AudioEngine/AudioEngine.h>
 #include <core/Hydrogen.h>
 #include <core/Globals.h>
 #include <core/EventQueue.h>
-#include <core/Preferences.h>
+#include <core/Preferences/Preferences.h>
 #include <core/H2Exception.h>
 #include <core/Basics/Playlist.h>
 #include <core/Helpers/Filesystem.h>
 #include <core/Helpers/Translations.h>
+#include <core/Logger.h>
 
 #ifdef H2CORE_HAVE_OSC
 #include <core/NsmClient.h>
@@ -62,73 +67,13 @@
 #include <map>
 #include <set>
 
-//
-// Set the palette used in the application
-//
-void setPalette( QApplication *pQApp )
-{
-	// create the default palette
-	QPalette defaultPalette;
+#ifdef HAVE_EXECINFO_H
+#include <execinfo.h>
+#endif
 
-	// A general background color.
-	defaultPalette.setColor( QPalette::Window, QColor( 58, 62, 72 ) );
-
-	// A general foreground color.
-	defaultPalette.setColor( QPalette::WindowText, QColor( 255, 255, 255 ) );
-
-	// Used as the background color for text entry widgets; usually white or another light color.
-	defaultPalette.setColor( QPalette::Base, QColor( 88, 94, 112 ) );
-
-	// Used as the alternate background color in views with alternating row colors
-	defaultPalette.setColor( QPalette::AlternateBase, QColor( 138, 144, 162 ) );
-
-	// The foreground color used with Base. This is usually the same as the Foreground, in which case it must provide good contrast with Background and Base.
-	defaultPalette.setColor( QPalette::Text, QColor( 255, 255, 255 ) );
-
-	// The general button background color. This background can be different from Background as some styles require a different background color for buttons.
-	defaultPalette.setColor( QPalette::Button, QColor( 88, 94, 112 ) );
-
-	// A foreground color used with the Button color.
-	defaultPalette.setColor( QPalette::ButtonText, QColor( 255, 255, 255 ) );
-
-
-	// Lighter than Button color.
-	defaultPalette.setColor( QPalette::Light, QColor( 138, 144, 162 ) );
-
-	// Between Button and Light.
-	defaultPalette.setColor( QPalette::Midlight, QColor( 128, 134, 152 ) );
-
-	// Darker than Button.
-	defaultPalette.setColor( QPalette::Dark, QColor( 58, 62, 72 ) );
-
-	// Between Button and Dark.
-	defaultPalette.setColor( QPalette::Mid, QColor( 81, 86, 99 ) );
-
-	// A very dark color. By default, the shadow color is Qt::black.
-	defaultPalette.setColor( QPalette::Shadow, QColor( 255, 255, 255 ) );
-
-
-	// A color to indicate a selected item or the current item.
-	defaultPalette.setColor( QPalette::Highlight, QColor( 116, 124, 149 ) );
-
-	// A text color that contrasts with Highlight.
-	defaultPalette.setColor( QPalette::HighlightedText, QColor( 255, 255, 255 ) );
-
-
-	// Desaturate disabled widgets by blending with the alternate colour
-	for ( QPalette::ColorRole role : { QPalette::Window, QPalette::Base, QPalette::AlternateBase, QPalette::Dark,
-									  QPalette::Light, QPalette::Midlight, QPalette::Mid, QPalette::Shadow,
-									  QPalette::Text } ) {
-		QColor normalColor = defaultPalette.color( QPalette::Normal, role );
-		QColor disabledColor = QColor( ( normalColor.red() + 138 ) / 2,
-									   ( normalColor.green() + 144 ) / 2,
-									   ( normalColor.blue() + 162 ) / 2);
-		defaultPalette.setColor( QPalette::Disabled, role, disabledColor );
-	}
-
-	pQApp->setPalette( defaultPalette );
-	pQApp->setStyleSheet("QToolTip {padding: 1px; border: 1px solid rgb(199, 202, 204); background-color: rgb(227, 243, 252); color: rgb(64, 64, 66);}");
-}
+namespace H2Core {
+	void init_gui_object_map();
+};
 
 // Handle a fatal signal, allowing the logger to complete any outstanding messages before re-raising the
 // signal to allow normal termination.
@@ -137,10 +82,27 @@ static void handleFatalSignal( int nSignal )
 	// First disable signal handler to allow normal termination
 	signal( nSignal, SIG_DFL );
 
+	// Report current context to the crash reporter
+	Reporter::report();
+
+	___ERRORLOG( QString( "Fatal signal %1" ).arg( nSignal ) );
+
+#ifdef HAVE_EXECINFO_H
+	// Print out stack backtrace if we can
+	const int nMaxFrames = 128;
+	void *frames[ nMaxFrames ];
+	int nFrames = backtrace( frames, nMaxFrames );
+	char **symbols = backtrace_symbols( frames, nFrames );
+	for ( int i = 0; i < nFrames; i++ ) {
+		___ERRORLOG( QString("%1").arg( symbols[i] ) );
+	}
+#endif
+
 	// Allow logger to complete
 	H2Core::Logger* pLogger = H2Core::Logger::get_instance();
-	if ( pLogger )
+	if ( pLogger ) {
 		delete pLogger;
+	}
 
 	raise( nSignal );
 }
@@ -159,12 +121,16 @@ static int setup_unix_signal_handlers()
 		return 1;
 	}
 
-	for ( int nSignal : { SIGSEGV, SIGILL, SIGFPE, SIGBUS } ) {
+#endif
+
+	for ( int nSignal : { SIGSEGV, SIGILL, SIGFPE, SIGABRT,
+#ifndef WIN32
+						 SIGBUS, SIGTRAP
+#endif
+		} ) {
 		signal( nSignal, handleFatalSignal );
 	}
 
-#endif
-	
 	return 0;
 }
 
@@ -227,9 +193,21 @@ public:
 
 int main(int argc, char *argv[])
 {
+
+#ifdef WIN32
+	// In case Hydrogen was started using a CLI attach its output to
+	// the latter. 
+	if ( AttachConsole(ATTACH_PARENT_PROCESS)) {
+		freopen("CONOUT$", "w", stdout);
+		freopen("CONOUT$", "w", stderr);
+		freopen("CONIN$", "w", stdin);
+	}
+#endif
+	Reporter::spawn( argc, argv );
 	try {
 #if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
 		QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+		QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
 #endif
 
 		// Create bootstrap QApplication to get H2 Core set up with correct Filesystem paths before starting GUI application.
@@ -240,21 +218,46 @@ int main(int argc, char *argv[])
 		QCommandLineParser parser;
 		
 		QString aboutText = QString( "\nHydrogen " ) + QString::fromStdString( H2Core::get_version() )  + QString( " [" ) + QString::fromStdString( __DATE__ ) + QString( "]  [http://www.hydrogen-music.org]" ) +
-		QString( "\nCopyright 2002-2008 Alessandro Cominu\nCopyright 2008-2021 The hydrogen development team" ) +
+		QString( "\nCopyright 2002-2008 Alessandro Cominu\nCopyright 2008-2023 The hydrogen development team" ) +
 		QString( "\nHydrogen comes with ABSOLUTELY NO WARRANTY\nThis is free software, and you are welcome to redistribute it under certain conditions. See the file COPYING for details.\n" );
 		
 		parser.setApplicationDescription( aboutText );
+
+		QStringList availableAudioDrivers;
+#ifdef H2CORE_HAVE_JACK
+		availableAudioDrivers << "jack";
+#endif
+#ifdef H2CORE_HAVE_ALSA
+		availableAudioDrivers << "alsa";
+#endif
+#ifdef H2CORE_HAVE_OSS
+		availableAudioDrivers << "oss";
+#endif
+#ifdef H2CORE_HAVE_PULSEAUDIO
+		availableAudioDrivers << "pulseaudio";
+#endif
+#ifdef H2CORE_HAVE_PORTAUDIO
+		availableAudioDrivers << "portaudio";
+#endif
+#ifdef H2CORE_HAVE_COREAUDIO
+		availableAudioDrivers << "coreaudio";
+#endif
+		availableAudioDrivers << "auto";
 		
-		QCommandLineOption audioDriverOption( QStringList() << "d" << "driver", "Use the selected audio driver (jack, alsa, oss)", "Audiodriver");
+		QCommandLineOption audioDriverOption( QStringList() << "d" << "driver",
+											  QString( "Use the selected audio driver (%1)" )
+											  .arg( availableAudioDrivers.join( ", " ) ),
+											  "Audiodriver");
 		QCommandLineOption installDrumkitOption( QStringList() << "i" << "install", "Install a drumkit (*.h2drumkit)" , "File");
 		QCommandLineOption noSplashScreenOption( QStringList() << "n" << "nosplash", "Hide splash screen" );
 		QCommandLineOption playlistFileNameOption( QStringList() << "p" << "playlist", "Load a playlist (*.h2playlist) at startup", "File" );
 		QCommandLineOption systemDataPathOption( QStringList() << "P" << "data", "Use an alternate system data path", "Path" );
 		QCommandLineOption songFileOption( QStringList() << "s" << "song", "Load a song (*.h2song) at startup", "File" );
 		QCommandLineOption kitOption( QStringList() << "k" << "kit", "Load a drumkit at startup", "DrumkitName" );
-		QCommandLineOption verboseOption( QStringList() << "V" << "verbose", "Level, if present, may be None, Error, Warning, Info, Debug or 0xHHHH","Level");
+		QCommandLineOption verboseOption( QStringList() << "V" << "verbose", "Level, if present, may be None, Error, Warning, Info, Debug, Constructors, Locks, or 0xHHHH", "Level" );
 		QCommandLineOption shotListOption( QStringList() << "t" << "shotlist", "Shot list of widgets to grab", "ShotList" );
 		QCommandLineOption uiLayoutOption( QStringList() << "layout", "UI layout ('tabbed' or 'single')", "Layout" );
+		QCommandLineOption noReporterOption( QStringList() << "child", "Child process (no crash reporter)");
 		
 		parser.addHelpOption();
 		parser.addVersionOption();
@@ -268,14 +271,9 @@ int main(int argc, char *argv[])
 		parser.addOption( verboseOption );
 		parser.addOption( shotListOption );
 		parser.addOption( uiLayoutOption );
+		parser.addOption( noReporterOption );
 		parser.addPositionalArgument( "file", "Song, playlist or Drumkit file" );
 		
-		//Conditional options
-		#ifdef H2CORE_HAVE_JACKSESSION
-			QCommandLineOption jackSessionOption(QStringList() << "S" << "jacksessionid", "ID - Start a JackSessionHandler session");
-			parser.addOption(jackSessionOption);
-		#endif
-			
 		// Evaluate the options
 		parser.process( *pBootStrApp );
 		QString sSelectedDriver = parser.value( audioDriverOption );
@@ -316,19 +314,17 @@ int main(int argc, char *argv[])
 			}
 		}
 		
-		#ifdef H2CORE_HAVE_JACKSESSION
-				QString sessionId;
-		#endif
-		
 		std::cout << aboutText.toStdString();
 		
 		setup_unix_signal_handlers();
+		QString sInitialisingCrashContext( "Initialising Hydrogen" );
+		H2Core::Logger::setCrashContext( &sInitialisingCrashContext );
 
 		// Man your battle stations... this is not a drill.
 		H2Core::Logger::create_instance();
 		H2Core::Logger::set_bit_mask( logLevelOpt );
 		H2Core::Logger* pLogger = H2Core::Logger::get_instance();
-		H2Core::Object::bootstrap( pLogger, pLogger->should_log(H2Core::Logger::Debug) );
+		H2Core::Base::bootstrap( pLogger, pLogger->should_log(H2Core::Logger::Debug) );
 		
 		if( sSysDataPath.length() == 0 ) {
 			H2Core::Filesystem::bootstrap( pLogger );
@@ -350,13 +346,13 @@ int main(int argc, char *argv[])
 		Qt::HighDpiScaleFactorRoundingPolicy policy;
 
 		switch ( pPref->getUIScalingPolicy() ) {
-		case H2Core::Preferences::UI_SCALING_SMALLER:
+		case H2Core::InterfaceTheme::ScalingPolicy::Smaller:
 			policy = Qt::HighDpiScaleFactorRoundingPolicy::RoundPreferFloor;
 			break;
-		case H2Core::Preferences::UI_SCALING_SYSTEM:
+		case H2Core::InterfaceTheme::ScalingPolicy::System:
 			policy = Qt::HighDpiScaleFactorRoundingPolicy::PassThrough;
 			break;
-		case H2Core::Preferences::UI_SCALING_LARGER:
+		case H2Core::InterfaceTheme::ScalingPolicy::Larger:
 			policy = Qt::HighDpiScaleFactorRoundingPolicy::Ceil;
 			break;
 		}
@@ -366,9 +362,9 @@ int main(int argc, char *argv[])
 		// Force layout
 		if ( !sUiLayout.isEmpty() ) {
 			if ( sUiLayout == "tabbed" ) {
-				pPref->setDefaultUILayout( 1 );
+				pPref->setDefaultUILayout( H2Core::InterfaceTheme::Layout::Tabbed );
 			} else {
-				pPref->setDefaultUILayout( 0 );
+				pPref->setDefaultUILayout( H2Core::InterfaceTheme::Layout::SinglePane );
 			}
 		}
 
@@ -383,18 +379,26 @@ int main(int argc, char *argv[])
 			exit(0);
 		}
 		
-		if (sSelectedDriver == "auto") {
+		if ( sSelectedDriver == "auto" ) {
+			pPref->m_sAudioDriver = "Auto";
+		} else if ( sSelectedDriver == "jack" ) {
+			pPref->m_sAudioDriver = "JACK";
+		} else if ( sSelectedDriver == "oss" ) {
+			pPref->m_sAudioDriver = "OSS";
+		} else if ( sSelectedDriver == "alsa" ) {
+			pPref->m_sAudioDriver = "ALSA";
+		} else if ( sSelectedDriver == "pulseaudio" ) {
+			pPref->m_sAudioDriver = "PulseAudio";
+		} else if ( sSelectedDriver == "coreaudio" ) {
+			pPref->m_sAudioDriver = "CoreAudio";
+		} else if ( sSelectedDriver == "portaudio" ) {
+			pPref->m_sAudioDriver = "PortAudio";
+		} else if ( ! sSelectedDriver.isEmpty() ) {
+			___WARNINGLOG( QString( "Unknown driver [%1]. The 'auto' driver will be used instead" )
+						.arg( sSelectedDriver ) );
 			pPref->m_sAudioDriver = "Auto";
 		}
-		else if (sSelectedDriver == "jack") {
-			pPref->m_sAudioDriver = "JACK";
-		}
-		else if ( sSelectedDriver == "oss" ) {
-			pPref->m_sAudioDriver = "OSS";
-		}
-		else if ( sSelectedDriver == "alsa" ) {
-			pPref->m_sAudioDriver = "ALSA";
-		}
+				
 
 		// Bootstrap is complete, start GUI
 		delete pBootStrApp;
@@ -407,7 +411,7 @@ int main(int argc, char *argv[])
 		pQApp->processEvents();
 
 		QString family = pPref->getApplicationFontFamily();
-		pQApp->setFont( QFont( family, pPref->getApplicationFontPointSize() ) );
+		pQApp->setFont( QFont( family, 10 ) );
 
 		QTranslator qttor( nullptr );
 		QTranslator tor( nullptr );
@@ -445,7 +449,7 @@ int main(int argc, char *argv[])
 			pQApp->setStyle( sStyle );
 		}
 
-		setPalette( pQApp );
+		Skin::setPalette( pQApp );
 		setApplicationIcon(pQApp);
 
 		SplashScreen *pSplash = new SplashScreen();
@@ -491,52 +495,14 @@ int main(int argc, char *argv[])
 		}
 #endif
 
-#ifdef H2CORE_HAVE_JACKSESSION
-		if(!sessionId.isEmpty()){
-			pPref->setJackSessionUUID( sessionId );
-
-			/*
-			 * imo, jack sessions use jack as default audio driver.
-			 * hydrogen remember last used audiodriver.
-			 * here we make it save that hydrogen start in a jacksession case
-			 * every time with jack as audio driver
-			 */
-			pPref->m_sAudioDriver = "JACK";
-
-		}
-
-		/*
-		 * the use of applicationFilePath() make it
-		 * possible to use different executables.
-		 * for example if you start hydrogen from a local
-		 * build directory.
-		 */
-
-		QString path = pQApp->applicationFilePath();
-		pPref->setJackSessionApplicationPath( path );
-#endif
-
 		// Hydrogen here to honor all preferences.
 		H2Core::Hydrogen::create_instance();
+		auto pHydrogen = H2Core::Hydrogen::get_instance();
 		
 		// Tell Hydrogen it was started via the QT5 GUI.
-		H2Core::Hydrogen::get_instance()->setGUIState( H2Core::Hydrogen::GUIState::notReady );
+		pHydrogen->setGUIState( H2Core::Hydrogen::GUIState::notReady );
 		
-		// Whether or not to load a default song or supplied one when
-		// constructing the MainForm object.
-		bool bLoadSong = true;
-
-		H2Core::Hydrogen::get_instance()->startNsmClient();
-		
-		if ( H2Core::Hydrogen::get_instance()->isUnderSessionManagement() ){
-			
-			// When using the Non Session Management system, the new
-			// Song will be loaded by the NSM client singleton itself
-			// and not by the MainForm. The latter will just access
-			// the already loaded Song.
-			bLoadSong = false;
-			
-		}
+		pHydrogen->startNsmClient();
 
 		// If the NSM_URL variable is present, Hydrogen will not
 		// initialize the audio driver and leaves this to the callback
@@ -544,21 +510,22 @@ int main(int argc, char *argv[])
 		// called by now). However, the presence of the environmental
 		// variable does not guarantee for a session management and if
 		// no audio driver is initialized yet, we will do it here. 
-		if ( H2Core::Hydrogen::get_instance()->getAudioOutput() == nullptr ) {
+		if ( pHydrogen->getAudioOutput() == nullptr ) {
 			// Starting drivers can take some time, so show the wait cursor to let the user know that, yes,
 			// we're definitely busy.
 			QApplication::setOverrideCursor( Qt::WaitCursor );
-			H2Core::Hydrogen::get_instance()->restartDrivers();
+			pHydrogen->restartDrivers();
 			QApplication::restoreOverrideCursor();
 		}
 
-		MainForm *pMainForm = new MainForm( pQApp, sSongFilename, bLoadSong );
+		MainForm *pMainForm = new MainForm( pQApp, sSongFilename );
+		auto pHydrogenApp = HydrogenApp::get_instance();
 		pMainForm->show();
 		
 		pSplash->finish( pMainForm );
 
 		if( ! sPlaylistFilename.isEmpty() ){
-			bool loadlist = HydrogenApp::get_instance()->getPlayListDialog()->loadListByFileName( sPlaylistFilename );
+			bool loadlist = pHydrogenApp->getPlayListDialog()->loadListByFileName( sPlaylistFilename );
 			if ( loadlist ){
 				H2Core::Playlist::get_instance()->setNextSongByNumber( 0 );
 			} else {
@@ -566,41 +533,65 @@ int main(int argc, char *argv[])
 			}
 		}
 
+
 		if( ! sDrumkitToLoad.isEmpty() ) {
-			H2Core::Drumkit* pDrumkitInfo = H2Core::Drumkit::load_by_name( sDrumkitToLoad, true );
-			if ( pDrumkitInfo ) {
-				H2Core::Hydrogen::get_instance()->loadDrumkit( pDrumkitInfo );
-				HydrogenApp::get_instance()->onDrumkitLoad( pDrumkitInfo->get_name() );
-			} else {
-				___ERRORLOG ( "Error loading the drumkit" );
-			}
-			delete pDrumkitInfo;
+			pHydrogen->getCoreActionController()->setDrumkit( sDrumkitToLoad );
 		}
 
+		// Write the changes in the Preferences to disk to make them
+		// accessible in the PreferencesDialog.
+		pPref->savePreferences();
+
 		pQApp->setMainForm( pMainForm );
 
 		// Tell the core that the GUI is now fully loaded and ready.
-		H2Core::Hydrogen::get_instance()->setGUIState( H2Core::Hydrogen::GUIState::ready );
-#ifdef H2CORE_HAVE_OSC
-		if ( NsmClient::get_instance() != nullptr ) {
-			NsmClient::get_instance()->sendDirtyState( false );
-		}
-#endif
+		pHydrogen->setGUIState( H2Core::Hydrogen::GUIState::ready );
 
 		if ( sShotList != QString() ) {
 			ShotList *sl = new ShotList( sShotList );
 			sl->shoot();
 		}
 
+		// All GUI setup is complete, any spurious widget-driven
+		// flagging of song modified state will be complete, so clear
+		// the modification flag. This does not apply in case we are
+		// restoring unsaved changes applied to an empty song during
+		// the previous session.
+		if ( pHydrogen->getSong()->getFilename() !=
+			 H2Core::Filesystem::empty_song_path() ) {
+#ifdef H2CORE_HAVE_OSC
+			// Mark empty song created in a new NSM session modified
+			// in order to emphasis that an initial song save is
+			// required to generate the song file and link the
+			// associated drumkit in the session folder.
+			if ( NsmClient::get_instance() != nullptr &&
+				 NsmClient::get_instance()->getIsNewSession() ) {
+				
+				NsmClient::get_instance()->sendDirtyState( true );
+				pHydrogen->setIsModified( true );
+			}
+			else {
+				NsmClient::get_instance()->sendDirtyState( false );
+				pHydrogen->setIsModified( false );
+			}
+#else
+			pHydrogen->setIsModified( false );
+#endif
+		}
+
+		H2Core::Logger::setCrashContext( nullptr );
+
 		pQApp->exec();
 
+		QString sShutdownCrashContext( "Shutting down Hydrogen" );
+		H2Core::Logger::setCrashContext( &sShutdownCrashContext );
+
 		pPref->savePreferences();
 		delete pSplash;
 		delete pMainForm;
 		delete pQApp;
 		delete pPref;
 		delete H2Core::EventQueue::get_instance();
-		delete H2Core::AudioEngine::get_instance();
 
 		delete MidiMap::get_instance();
 		delete MidiActionManager::get_instance();
@@ -609,8 +600,8 @@ int main(int argc, char *argv[])
 		std::cout << "\nBye..." << std::endl;
 		delete H2Core::Logger::get_instance();
 
-		if (H2Core::Object::count_active()) {
-			H2Core::Object::write_objects_map_to_cerr();
+		if (H2Core::Base::count_active()) {
+			H2Core::Base::write_objects_map_to_cerr();
 		}
 
 	}
